<?php

namespace FacturareOnline\Inc\Admin;

use FacturareOnline\Inc\Libraries\MC;

if (!defined('ABSPATH')) {
	exit;
} // Exit if accessed directly

/**
 * @link              https://facturare.online
 * @since             1.0.0
 * @package           FacturareOnline
 *
 */
class Admin
{

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $plugin_name The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $version The current version of this plugin.
	 */
	private $version;

	/**
	 * The text domain of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $plugin_text_domain The text domain of this plugin.
	 */
	private $plugin_text_domain;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @param string $plugin_name The name of this plugin.
	 * @param string $version The version of this plugin.
	 * @param string $plugin_text_domain The text domain of this plugin.
	 *
	 * @since       1.0.0
	 *
	 */
	public function __construct($plugin_name, $version, $plugin_text_domain)
	{
		$this->plugin_name = $plugin_name;
		$this->version = $version;
		$this->plugin_text_domain = $plugin_text_domain;
	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles()
	{
		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */
		wp_enqueue_style($this->plugin_name . '-jquery-confirm', plugin_dir_url(__FILE__) . 'css/jquery-confirm.min.css', array(), $this->version, 'all');
		wp_enqueue_style($this->plugin_name, plugin_dir_url(__FILE__) . 'css/managementcompanie-admin.css', array(), $this->version, 'all');
	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts()
	{
		wp_register_script($this->plugin_name . '-jquery-confirm', plugin_dir_url(__FILE__) . 'js/jquery-confirm.min.js', array('jquery'), $this->version, true);
		wp_enqueue_script($this->plugin_name . '-jquery-confirm');
		wp_enqueue_script( 'jquery-ui-datepicker' );
		wp_register_script($this->plugin_name . '-admin-ajax-script', plugin_dir_url(__FILE__) . 'js/managementcompanie-admin.js', array('jquery', 'wp-i18n'), $this->version, true);
		wp_set_script_translations($this->plugin_name . '-admin-ajax-script', 'management-companie');
		wp_localize_script($this->plugin_name . '-admin-ajax-script', 'mc', array('ajaxurl' => admin_url('admin-ajax.php')));
		wp_enqueue_script($this->plugin_name . '-admin-ajax-script');

		wp_register_style( 'jquery-ui', 'https://code.jquery.com/ui/1.12.1/themes/smoothness/jquery-ui.css' );
		wp_enqueue_style( 'jquery-ui' );
	}

	public function add_meta_box_mc()
	{
		global $post;
		if (get_post_type($post) === 'shop_order') {
			add_meta_box('form_managementcompanie', \sprintf(__('Facturare.Online  - Order ID #%s', 'management-companie'), $post->ID), array($this, 'form_managementcompanie'), 'shop_order');
		}
	}

	public function form_managementcompanie($post)
	{
		load_template(plugin_dir_path(__FILE__) . 'views/html-managementcompanie-admin-display.php');
	}

	public function public_factura($order_id)
	{
		global $wpdb;
		$factura = $wpdb->get_row(
			"SELECT
                f.*,
                concat('[',
                GROUP_CONCAT(
                CONCAT( '{\"receiptid\":\"', c.receiptid, '\", \"serie_chitanta\":\"', c.serie, '\", \"numar_chitanta\":\"', c.numar, '\", \"total_chitanta\":\"', c.total,'\", \"moneda_chitanta\":\"', c.moneda,'\", \"created_on_chitanta\":\"', c.created_on, '\"}' ) order by c.id desc),
                 ']') AS chitante 
            FROM
                {$wpdb->prefix}managementcompanie_facturi f
                LEFT JOIN {$wpdb->prefix}managementcompanie_chitante c ON f.invoiceid = c.invoiceid 
            WHERE
                f.orderid = {$order_id} 
                AND f.status in (1,4,5)
            GROUP BY
                f.id 
            ORDER BY
                f.id DESC"
		);

		if (!empty($factura)):
			echo '<div style="background: #eeeeee;padding:10px;border:1px solid #cccccc;margin-bottom:10px">';
			echo \sprintf(__('The invoice Series <b>%s</b>, Number <b>%s</b>, was issued for this order at <b>%s</b>', 'management-companie'), $factura->serie, $factura->numar, $factura->created_on) . '</div>';
			?>
            <table style="width:100%">
                <thead>
                <tr>
                    <th style="text-align: center"><?php echo __('Type', 'management-companie'); ?></th>
                    <th style="text-align: center"><?php echo __('Series', 'management-companie'); ?></th>
                    <th style="text-align: center"><?php echo __('Number', 'management-companie'); ?></th>
                    <th style="text-align: center"><?php echo __('Total', 'management-companie'); ?></th>
                    <th style="text-align: center"><?php echo __('Generated at', 'management-companie'); ?></th>
                    <th style="text-align: center"><?php echo __('Actions', 'management-companie'); ?></th>
                </tr>
                <tbody>

                <tr style="background: #eee">
                    <td style="text-align: center"><?php echo MC::translateType($factura->tip) ?></td>
                    <td style="text-align: center"><?php echo $factura->serie ?></td>
                    <td style="text-align: center"><?php echo $factura->numar ?></td>
                    <td style="text-align: center"><?php echo $factura->total . ' ' . $factura->moneda ?></td>
                    <td style="text-align: center"><?php echo date('d-m-Y H:i', strtotime($factura->created_on)) ?></td>
                    <td style="text-align: center" width="20%">
                        <a href="<?php echo admin_url('admin-ajax.php') . '?' . http_build_query(array('action' => 'descarca_factura_public', 'order' => $order_id, 'invoiceid' => (int)$factura->invoiceid)) ?>"
                           class="download-invoice button">
							<?php echo __('Download', 'management-companie') ?>
                        </a>
                    </td>
                </tr>

				<?php
				$chitante = json_decode($factura->chitante);
				if (!empty($chitante)) :
					foreach ($chitante as $chitanta):
						echo '<tr>'; ?>
                        <td style="text-align: center"><?php echo __('Receipt', 'management-companie') ?></td>
                        <td style="text-align: center"><?php echo $chitanta->serie_chitanta ?></td>
                        <td style="text-align: center"><?php echo $chitanta->numar_chitanta ?></td>
                        <td style="text-align: center"><?php echo $chitanta->total_chitanta . ' ' . $chitanta->moneda_chitanta ?></td>
                        <td style="text-align: center"><?php echo date('d-m-Y H:i', strtotime($chitanta->created_on_chitanta)) ?></td>
                        <td style="text-align: center">
                            <a href="<?php echo admin_url('admin-ajax.php') . '?' . http_build_query(array('action' => 'descarca_chitanta_public', 'order' => $order_id, 'receiptid' => (int)$chitanta->receiptid)) ?>"
                               class="download-receipt button">
								<?php echo __('Download', 'management-companie') ?>
                            </a>
                        </td>
						<?php echo '</tr>';
					endforeach;
				endif; ?>
                </tbody>
                </thead>
            </table>
		<?php
		endif;
	}

	public function descarca_factura_public()
	{
		global $wpdb;
		if (empty($_GET['invoiceid'])) {
			wp_die('No invoiceid sent');
		}
		if (empty($_GET['order'])) {
			wp_die('No post sent');
		}
		$order_id = (int)sanitize_text_field($_GET['order']);

		if (get_post_type($order_id) !== 'shop_order') {
			wp_die(__('The post is not an Woocommerce order', 'management-companie'));
		}

		$order = wc_get_order($order_id);
		$order_user_id = $order->get_user_id();
		$loggedin_user_id = \get_current_user_id();

		if ($order_user_id != $loggedin_user_id) {
			wp_die(__('You are not allowed to download this resource', 'management-companie'));
		}

		$table_name = $wpdb->prefix . 'managementcompanie_facturi';
		$result = $wpdb->get_row("SELECT * from {$table_name} where invoiceid = " . (int)sanitize_text_field($_GET['invoiceid']));

		if (empty($result)) {
			wp_die(__('This resource is unavailable', 'management-companie'));
		}

		$decoded = base64_decode($result->document);
		switch ($result->tip) {
			case 'ff':
				$prefix = 'factura-fiscala-';
				break;
			case 'fp':
				$prefix = 'factura-proforma-';
				break;
			case 'c':
				$prefix = 'chitanta-';
				break;
		}
		$file = $prefix . $result->serie . '-' . $result->numar . '.pdf';

		file_put_contents($file, $decoded);

		if (file_exists($file)) {
			header('Content-Description: File Transfer');
			header('Content-Type: application/pdf');
			header('Content-Disposition: attachment; filename="' . basename($file) . '"');
			header('Expires: 0');
			header('Cache-Control: must-revalidate');
			header('Pragma: public');
			header('Content-Length: ' . filesize($file));
			readfile($file);
			unlink($file);
			wp_die();
		}
		wp_die();
	}

	public function descarca_chitanta_public()
	{
		global $wpdb;
		if (empty($_GET['receiptid'])) {
			wp_die('No invoiceid sent');
		}
		if (empty($_GET['order'])) {
			wp_die('No post sent');
		}
		$order_id = (int)sanitize_text_field($_GET['order']);

		if (get_post_type($order_id) !== 'shop_order') {
			$this->response('error', __('The post is not an Woocommerce order', 'management-companie'));
			wp_die();
		}

		$order = wc_get_order($order_id);
		$order_user_id = $order->get_user_id();
		$loggedin_user_id = \get_current_user_id();

		if ($order_user_id != $loggedin_user_id) {
			wp_die(__('You are not allowed to download this resource', 'management-companie'));
		}

		$table_name = $wpdb->prefix . 'managementcompanie_chitante';
		$result = $wpdb->get_row("SELECT * from {$table_name} where receiptid = " . (int)sanitize_text_field($_GET['receiptid']));

		if (empty($result)) {
			wp_die(__('This resource is unavailable', 'management-companie'));
		}

		$decoded = base64_decode($result->document);
		$file = 'chitanta-' . $result->serie . '-' . $result->numar . '.pdf';

		file_put_contents($file, $decoded);

		if (file_exists($file)) {
			header('Content-Description: File Transfer');
			header('Content-Type: application/pdf');
			header('Content-Disposition: attachment; filename="' . basename($file) . '"');
			header('Expires: 0');
			header('Cache-Control: must-revalidate');
			header('Pragma: public');
			header('Content-Length: ' . filesize($file));
			readfile($file);
			unlink($file);
			wp_die();
		}
		wp_die();
	}

	public function descarca_factura()
	{
		global $wpdb;
		if (empty($_GET['invoiceid'])) {
			wp_die('No invoiceid sent');
		}
		if (empty($_GET['order'])) {
			wp_die('No post sent');
		}
		$order_id = (int)sanitize_text_field($_GET['order']);

		if (get_post_type($order_id) !== 'shop_order') {
			$this->response('error', __('The post is not an Woocommerce order', 'management-companie'));
			wp_die();
		}

		$table_name = $wpdb->prefix . 'managementcompanie_facturi';
		$result = $wpdb->get_row("SELECT * from {$table_name} where invoiceid = " . (int)sanitize_text_field($_GET['invoiceid']));
		$decoded = base64_decode($result->document);
		switch ($result->tip) {
			case 'ff':
				$prefix = 'factura-fiscala-';
				break;
			case 'fp':
				$prefix = 'factura-proforma-';
				break;
			case 'c':
				$prefix = 'chitanta-';
				break;
		}
		$file = $prefix . $result->serie . '-' . $result->numar . '.pdf';

		file_put_contents($file, $decoded);

		if (file_exists($file)) {
			header('Content-Description: File Transfer');
			header('Content-Type: application/pdf');
			header('Content-Disposition: attachment; filename="' . basename($file) . '"');
			header('Expires: 0');
			header('Cache-Control: must-revalidate');
			header('Pragma: public');
			header('Content-Length: ' . filesize($file));
			readfile($file);
			unlink($file);
			wp_die();
		}
		wp_die();
	}

	public function descarca_chitanta()
	{
		global $wpdb;
		if (empty($_GET['receiptid'])) {
			wp_die('No invoiceid sent');
		}
		if (empty($_GET['order'])) {
			wp_die('No post sent');
		}
		$order_id = (int)sanitize_text_field($_GET['order']);

		if (get_post_type($order_id) !== 'shop_order') {
			$this->response('error', __('The post is not an Woocommerce order', 'management-companie'));
			wp_die();
		}

		$table_name = $wpdb->prefix . 'managementcompanie_chitante';
		$result = $wpdb->get_row("SELECT * from {$table_name} where receiptid = " . (int)sanitize_text_field($_GET['receiptid']));
		$decoded = base64_decode($result->document);
		$file = 'chitanta-' . $result->serie . '-' . $result->numar . '.pdf';

		file_put_contents($file, $decoded);

		if (file_exists($file)) {
			header('Content-Description: File Transfer');
			header('Content-Type: application/pdf');
			header('Content-Disposition: attachment; filename="' . basename($file) . '"');
			header('Expires: 0');
			header('Cache-Control: must-revalidate');
			header('Pragma: public');
			header('Content-Length: ' . filesize($file));
			readfile($file);
			unlink($file);
			wp_die();
		}
		wp_die();
	}

	private function response($status = 'error', $message = '')
	{
		echo json_encode(array('status' => $status, 'message' => __($message, 'management-companie')));
	}

	public function incaseaza_banca_factura()
	{
		global $wpdb;
		if (empty($_POST['invoiceid'])) {
			wp_die('No invoiceid sent');
		}
		if (empty($_POST['suma'])) {
			wp_die('No amount sent');
		}
		if (isset($_POST['suma']) && $_POST['suma'] < 0.0001) {
			wp_die('Amount not valid, must be greater than 0.0001');
		}
		if (empty($_POST['order'])) {
			wp_die('No post sent');
		}
		$order_id = (int)sanitize_text_field($_POST['order']);
		$date = sanitize_text_field($_POST['date']);

		if (get_post_type($order_id) !== 'shop_order') {
			$this->response('error', __('The post is not an Woocommerce order', 'management-companie'));
			wp_die();
		}

		$order = new \WC_Order($order_id);
		$exchange_rate = !empty($order->get_meta('_wcpbc_base_exchange_rate')) ? (float)$order->get_meta('_wcpbc_base_exchange_rate') : 1;

		$incaseaza_banca_factura = MC::incaseazaBancaFactura((int)sanitize_text_field($_POST['invoiceid']), (float)sanitize_text_field($_POST['suma']), $date, $exchange_rate);

		if (MC::isJson($incaseaza_banca_factura) === true) {
			if (MC::isError(json_decode($incaseaza_banca_factura))) {
				echo $incaseaza_banca_factura;
				wp_die();
			} else {
				$incasare = json_decode($incaseaza_banca_factura);
				$message = json_decode($incasare->response->message);

				// am generat cu succes chitanta, o salvez in baza de date
				$table_name = $wpdb->prefix . 'managementcompanie_incasari_banca';

				$wpdb->insert($table_name, array(
					'invoiceid' => (int)$message->invoiceid,
					'amount'    => (float)$message->amount,
					'date'      => $message->date,
				));

				// actualizez starea facturii
				$table_name = $wpdb->prefix . 'managementcompanie_facturi';

				$wpdb->update($table_name, array(
					'status' => (int)$message->invoice_status,
				), array(
					'invoiceid' => (int)$message->invoiceid,
				));

				echo $incaseaza_banca_factura;
				wp_die();
			}
		} else {
			echo MC::response('', '01', __('The message from server response is not JSON', 'management-companie'));
			wp_die();
		}
		wp_die();
	}

	public function genereaza_chitanta_factura()
	{
		global $wpdb;
		if (empty($_POST['invoiceid'])) {
			wp_die('No invoiceid sent');
		}
		if (empty($_POST['suma'])) {
			wp_die('No amount sent');
		}
		if (isset($_POST['suma']) && $_POST['suma'] < 0.0001) {
			wp_die('Amount not valid, must be greater than 0.0001');
		}
		if (empty($_POST['order'])) {
			wp_die('No post sent');
		}
		$order_id = (int)sanitize_text_field($_POST['order']);

		if (get_post_type($order_id) !== 'shop_order') {
			$this->response('error', __('The post is not an Woocommerce order', 'management-companie'));
			wp_die();
		}

		$generare_chitanta_factura = MC::genereazaChitantaFactura((int)sanitize_text_field($_POST['invoiceid']), (float)sanitize_text_field($_POST['suma']));

		if (MC::isJson($generare_chitanta_factura) === true) {
			if (MC::isError(json_decode($generare_chitanta_factura))) {
				echo $generare_chitanta_factura;
				wp_die();
			} else {
				$chitanta = json_decode($generare_chitanta_factura);
				$message = json_decode($chitanta->response->message);

				// am generat cu succes chitanta, o salvez in baza de date
				$table_name = $wpdb->prefix . 'managementcompanie_chitante';

				$wpdb->insert($table_name, array(
					'receiptid' => (int)$message->receiptid,
					'invoiceid' => (int)$message->invoiceid,
					'serie'     => $message->serie,
					'total'     => (float)$message->total,
					'moneda'    => $message->moneda,
					'numar'     => (int)$message->numar,
					'code'      => $message->code,
					'document'  => $message->document,
				));

				// actualizez starea facturii
				$table_name = $wpdb->prefix . 'managementcompanie_facturi';

				$wpdb->update($table_name, array(
					'status' => (int)$message->invoice_status,
				), array(
					'invoiceid' => (int)$message->invoiceid,
				));

				echo $generare_chitanta_factura;
				wp_die();
			}
		} else {
			echo MC::response('', '01', __('The message from server response is not JSON', 'management-companie'));
			wp_die();
		}
		wp_die();
	}

	public function anuleaza_factura()
	{
		global $wpdb;
		if (empty($_POST['invoiceid'])) {
			wp_die('No invoiceid sent');
		}
		if (empty($_POST['order'])) {
			wp_die('No post sent');
		}
		$order_id = (int)sanitize_text_field($_POST['order']);

		if (get_post_type($order_id) !== 'shop_order') {
			$this->response('error', __('The post is not an Woocommerce order', 'management-companie'));
			wp_die();
		}

		$anulare_factura = MC::anuleazaFactura((int)sanitize_text_field($_POST['invoiceid']));

		if (MC::isJson($anulare_factura) === true) {
			if (MC::isError(json_decode($anulare_factura))) {
				echo $anulare_factura;
				wp_die();
			} else {
				$factura = json_decode($anulare_factura);
				$message = json_decode($factura->response->message);

				// am anulat cu succes factura, o salvez in baza de date
				$table_name = $wpdb->prefix . 'managementcompanie_facturi';

				$wpdb->update($table_name, array(
					'status'   => (int)$message->status,
					'document' => $message->document,
				), array(
					'invoiceid' => (int)$message->invoiceid,
				));
				echo $anulare_factura;
				wp_die();
			}
		} else {
			echo MC::response('', '01', __('The message from server response is not JSON', 'management-companie'));
			wp_die();
		}
		wp_die();
	}

	public function vizualizeaza_chitanta()
	{
		global $wpdb;
		if (empty($_POST['receiptid'])) {
			wp_die('No invoiceid sent');
		}
		if (empty($_POST['order'])) {
			wp_die('No post sent');
		}
		$order_id = (int)sanitize_text_field($_POST['order']);

		if (get_post_type($order_id) !== 'shop_order') {
			$this->response('error', __('The post is not an Woocommerce order', 'management-companie'));
			wp_die();
		}

		$table_name = $wpdb->prefix . 'managementcompanie_chitante';
		$result = $wpdb->get_row("SELECT * from {$table_name} where receiptid = " . (int)sanitize_text_field($_POST['receiptid']));

		$file = 'chitanta-' . $result->serie . '-' . $result->numar . '.pdf';

		echo '<embed width="100%" height="100%" id="printSection" src="data:application/pdf;headers=filename%3D' . $file . ';base64,' . $result->document . '">';
		wp_die();
	}

	public function vizualizeaza_factura()
	{
		global $wpdb;
		if (empty($_POST['invoiceid'])) {
			wp_die('No invoiceid sent');
		}
		if (empty($_POST['order'])) {
			wp_die('No post sent');
		}
		$order_id = (int)sanitize_text_field($_POST['order']);

		if (get_post_type($order_id) !== 'shop_order') {
			$this->response('error', __('The post is not an Woocommerce order', 'management-companie'));
			wp_die();
		}

		$table_name = $wpdb->prefix . 'managementcompanie_facturi';
		$result = $wpdb->get_row("SELECT * from {$table_name} where invoiceid = " . (int)sanitize_text_field($_POST['invoiceid']));

		switch ($result->tip) {
			case 'ff':
				$prefix = 'factura-fiscala-';
				break;
			case 'fp':
				$prefix = 'factura-proforma-';
				break;
			case 'c':
				$prefix = 'chitanta-';
				break;
		}
		$file = $prefix . $result->serie . '-' . $result->numar . '.pdf';

		echo '<embed width="100%" height="100%" id="printSection" src="data:application/pdf;headers=filename%3D' . $file . ';base64,' . $result->document . '">';
		wp_die();
	}

	public function adauga_factura()
	{
		global $wpdb;
		if (empty($_POST['order'])) {
			wp_die('No post sent');
		}
		$order_id = (int)sanitize_text_field($_POST['order']);

		if (get_post_type($order_id) !== 'shop_order') {
			$this->response('error', __('The post is not an Woocommerce order', 'management-companie'));
			wp_die();
		}

		$emite_factura = MC::getFactura($order_id);
		if (MC::isJson($emite_factura) === true) {
			if (MC::isError(json_decode($emite_factura))) {
				echo $emite_factura;
				wp_die();
			} else {
				$factura = json_decode($emite_factura);

				if (MC::isJson($factura->response->message) === true) {
					$message = json_decode($factura->response->message);
				} else {
					echo MC::response('', '01', __('The message from server response is not JSON', 'management-companie'));
					wp_die();
				}

				// am emis cu succes factura, o salvez in baza de date
				$table_name = $wpdb->prefix . 'managementcompanie_facturi';

				$wpdb->insert($table_name, array(
					'orderid'   => (int)$message->orderid,
					'invoiceid' => (int)$message->invoiceid,
					'total'     => (float)$message->total,
					'moneda'    => $message->moneda,
					'serie'     => $message->serie,
					'numar'     => (int)$message->numar,
					'tip'       => $message->tip,
					'code'      => $message->code,
					'status'    => (int)$message->status,
					'document'  => $message->document,
				));
				echo $emite_factura;
				wp_die();
			}
		}

		wp_die();
	}

	public function add_company_fields($fields)
	{
		if (isset(get_option('managementcompanie_setari_generale')['permite_factura_persoana_juridica']) && get_option('managementcompanie_setari_generale')['permite_factura_persoana_juridica'] == 1) {
			$fields['billing']['company_cui'] = array(
				'label'    => __('Company CUI', 'management-companie'),
				'required' => false,
				'class'    => array('form-row-wide'),
				'clear'    => true,
				'priority' => 31,
			);

			$fields['billing']['company_regcom'] = array(
				'label'    => __('Company RegCom', 'management-companie'),
				'required' => false,
				'class'    => array('form-row-wide'),
				'clear'    => true,
				'priority' => 32,
			);
		} else {
			unset($fields['billing']['billing_company']);
		}
		return $fields;
	}

	function mc_customer_meta_fields($fields)
	{
		$fields['billing']['fields']['company_cui'] = array(
			'label'       => __('Company CUI', 'management-companie'),
			'description' => '',
			'required'    => false,
		);

		$fields['billing']['fields']['company_regcom'] = array(
			'label'       => __('Company RegCom', 'management-companie'),
			'description' => '',
			'required'    => false,
		);

		return $fields;
	}

	public function mc_woocommerce_billing_fields($fields)
	{
		$user_id = get_current_user_id();
		$user = get_userdata($user_id);

		if (!$user) {
			return $fields;
		}

		if (isset(get_option('managementcompanie_setari_generale')['permite_factura_persoana_juridica']) && get_option('managementcompanie_setari_generale')['permite_factura_persoana_juridica'] == 1) {
			$fields['company_cui'] = array(
				'label'    => __('Company CUI', 'management-companie'),
				'required' => false,
				'class'    => array('form-row-wide'),
				'clear'    => true,
				'priority' => 31,
			);

			$fields['company_regcom'] = array(
				'label'    => __('Company RegCom', 'management-companie'),
				'required' => false,
				'class'    => array('form-row-wide'),
				'clear'    => true,
				'priority' => 32,
			);
		} else {
			unset($fields['billing_company']);
		}

		return $fields;
	}

	function mc_display_order_data($order_id)
	{ ?>
		<?php if (isset(get_option('managementcompanie_setari_generale')['permite_factura_persoana_juridica']) && get_option('managementcompanie_setari_generale')['permite_factura_persoana_juridica'] == 1) : ?>
        <h2><?php _e(__('Company information', 'management-companie')); ?></h2>
        <table class="shop_table shop_table_responsive additional_info">
            <tbody>
            <tr>
                <th><?php _e(__('Company CUI', 'management-companie')); ?></th>
                <td><?php echo get_post_meta($order_id, 'company_cui', true); ?></td>
            </tr>
            <tr>
                <th><?php _e(__('Company RegCom', 'management-companie')); ?></th>
                <td><?php echo get_post_meta($order_id, 'company_regcom', true); ?></td>
            </tr>
            </tbody>
        </table>
	<?php endif;
	}

	public function mc_email_order_meta_fields($fields, $sent_to_admin, $order)
	{
		$fields['company_cui'] = array(
			'label' => __('Company CUI', 'management-companie'),
			'value' => get_post_meta($order->get_id(), 'company_cui', true),
		);
		$fields['company_regcom'] = array(
			'label' => __('Company RegCom', 'management-companie'),
			'value' => get_post_meta($order->get_id(), 'company_regcom', true),
		);
		return $fields;
	}

	public function save_company_fields($order_id)
	{
		if (!empty($_POST['company_cui'])) {
			update_post_meta($order_id, 'company_cui', sanitize_text_field($_POST['company_cui']));
		}
		if (!empty($_POST['company_regcom'])) {
			update_post_meta($order_id, 'company_regcom', sanitize_text_field($_POST['company_regcom']));
		}
	}

	public function check_company_fields($data, $errors)
	{
		if (!empty($data['billing_company'])) {
			if (empty($data['company_cui'])) {
				$errors->add('validation', __('You entered a company name, please fill in the Company CUI field', 'management-companie'));
			}
			if (empty($data['company_regcom'])) {
				$errors->add('validation', __('You entered a company name, please fill in the Company RegCom field', 'management-companie'));
			}
		}
	}

	public function create_alias_name_for_product()
	{
		$args = array(
			'id'          => 'mc_product_alias',
			'label'       => __('Facturare.Online alias name for product', 'management-companie'),
			'class'       => 'form-row-wide',
			'desc_tip'    => true,
			'description' => __('If set, this value will be used for invoice product name instead of Woocommerce product name', 'management-companie'),
		);
		woocommerce_wp_text_input($args);
	}

	function save_alias_name_for_product($post_id)
	{
		$product = wc_get_product($post_id);
		$title = isset($_POST['mc_product_alias']) ? $_POST['mc_product_alias'] : '';
		$product->update_meta_data('mc_product_alias', sanitize_text_field($title));
		$product->save();
	}


	public function snif()
	{
		global $wpdb;
		$input = file_get_contents('php://input');
		if (MC::isJson($input) === true) {
			$input = json_decode($input);

			$snif = MC::decryptMessageCompany($input->loginid, $input->message, $input->crypt_message);
			if (MC::isJson($snif) === true) {
				$snif = json_decode($snif);
				$table_name = $wpdb->prefix . 'managementcompanie_facturi';

				$result = $wpdb->update($table_name, array(
					'status'   => (int)$snif->status,
					'document' => $snif->document,
				), array(
					'invoiceid' => (int)$snif->invoiceid,
				));
				if ($result) {
					die(MC::response(array('invoiceid' => (int)$snif->invoiceid, 'status' => (int)$snif->status)));
				} else {
					die(MC::response('', '03', 'Could not update invoice in your system'));
				}
			} else {
				die(MC::response('', '02', 'The message from server response is not JSON'));
			}
		} else {
			die(MC::response('', '01', 'The message from server response is not JSON'));
		}
	}
}
